<?php
/**
 * ParseController
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 22.01.2015
 * @since 1.0.0
 */
namespace skeeks\modules\cms\kladr\controllers;

use skeeks\cms\base\console\Controller;
use skeeks\cms\models\User;
use skeeks\cms\modules\admin\components\UrlRule;
use skeeks\cms\modules\admin\controllers\AdminController;
use skeeks\cms\rbac\AuthorRule;
use skeeks\modules\cms\kladr\helpers\Autoru;
use skeeks\modules\cms\kladr\helpers\Updater;
use skeeks\modules\cms\kladr\models\City;
use skeeks\modules\cms\kladr\models\Country;
use skeeks\modules\cms\kladr\models\Metro;
use skeeks\modules\cms\kladr\models\Region;
use Yii;
use yii\db\ActiveQuery;
use yii\helpers\ArrayHelper;
use yii\helpers\Console;

class ParseController extends Controller
{
    /**
     * @return Country
     */
    protected function _fetchRussia()
    {
        $country = Country::find()->where(["code" => 'ru'])->one();
        $this->stdoutN("Find russia");

        if ($country)
        {
            $this->stdoutN("Russia is finded id: " . $country->id);
            return $country;
        }
        return null;
    }

    /*public function actionMetro()
    {
        $city = City::find()->where(["name" => "Москва"])->one();

        $data = \Yii::$app->db->createCommand('SELECT * FROM metro')->queryAll();
        foreach ($data as $data)
        {
            $metro = new Metro();
            $metro->name = $data['metro'];
            $metro->kladr_city_id = $city->id;

            $metro->save(false);
        }
    }*/
    /**
     */
    public function actionAutoruAll()
    {
        $this->stdoutN("Parsing data from auto.ru");

        $this->stdoutN();
        $this->stdoutN("Step 1");
        $this->stdoutN();

        $this->actionAutoruCountry();

        $this->stdoutN();
        $this->stdoutN("Step 2");
        $this->stdoutN();

        $this->actionAutoruRegionAndCity();

        $this->stdoutN("ready");
    }

    public function actionAutoruRegionAndCity()
    {
        $this->stdoutN("Updating regions and add/update cities");

        $regions = Region::find()->where(['not', ['autoru_id' => '']])->orderBy("autoru_id ASC")->all();
        $allFounded = count($regions);

        $this->stdoutN("Regions founded in database: {$allFounded}");

        Console::startProgress(0, $allFounded);

        foreach ($regions as $key => $region)
        {
            $citiesData = Autoru::getCities($region->autoru_id);
            $this->_updateRegionAndCities($citiesData, $region);
            usleep(2000);
            Console::updateProgress($key, $allFounded);
            usleep(1000);
        }

        Console::endProgress();

        return $this;

    }

    /**
     * @return $this
     */
    public function actionAutoruCountry()
    {
        $this->stdoutN("Adding countries and short data by regions");

        Console::startProgress(0, 100);

        for ($i = 1; $i <= 100; $i ++)
        {
            $regionsData = Autoru::getRegions($i);
            $this->_addCountryAndRegions($regionsData);

            usleep(2000);
            Console::updateProgress($i, 100);
        }

        Console::endProgress();

        return $this;
    }




    protected function _updateRegionAndCities($data, Region $region)
    {
        if (!$data)
        {
            return false;
        }


        if ($regionData = ArrayHelper::getValue($data, 'region'))
        {
            $this->stdoutN("Update region data: {$region->name}");

            $region->autoru_id     = $regionData['id'];
            $region->name          = $regionData['name'];
            $region->name_full     = $regionData['name_full'];
            $region->name_where    = $regionData['name_where'];
            $region->code          = $regionData['code'];
            $region->lat           = $regionData['coordinates_lat'];
            $region->lng           = $regionData['coordinates_lng'];
            $region->important     = $regionData['large'];

            $region->save(false);

        }

        $this->stdoutN("Update cities data: {$region->name}");
        if ($citiesData = ArrayHelper::getValue($data, 'cities'))
        {
            if ($citiesData)
            {
                foreach ($citiesData as $cityData)
                {
                    $this->_addCity($cityData, $region);
                }
            }
        }

        return $this;
    }


    protected function _addCountryAndRegions($data)
    {
        if (!$data)
        {
            return false;
        }

        $country = null;

        if ($countryData = ArrayHelper::getValue($data, 'country'))
        {
            $country = $this->_addCountry($countryData);
        }

        if (!$country)
        {
            $this->stdoutN("Country not added...");
            return false;
        }

        if ($regionsData = ArrayHelper::getValue($data, 'regions'))
        {
            if ($regionsData)
            {
                foreach ($regionsData as $regionData)
                {
                    $this->_addShortRegion($regionData, $country);
                }
            }
        }

        return $this;
    }


    protected function _addCity($data, Region $region)
    {
        $this->stdoutN("City: {$data['name']}");

        $city = City::find()->where(['autoru_id' => $data['id']])->one();

        if ($city)
        {
            $this->stdoutN("City already exist. Update info.");

            $city->autoru_id                 = $data['id'];
            $city->name                      = $data['name'];
            $city->name_full                 = $data['name_full'];
            $city->name_where                = $data['name_where'];
            $city->dialcode                  = $data['dialcode'];

            $city->alias                     = $data['alias'];

            $city->lat                       = $data['coordinates_lat'];
            $city->lng                       = $data['coordinates_lng'];
            $city->important                 = $data['popular'];

            $city->kladr_country_id          = $region->kladr_country_id;
            $city->kladr_region_id           = $region->id;

            $city->save(false);

        } else
        {
            $city = new City();

            $city->autoru_id                 = $data['id'];
            $city->name                      = $data['name'];
            $city->name_full                 = $data['name_full'];
            $city->name_where                = $data['name_where'];
            $city->dialcode                  = $data['dialcode'];

            $city->alias                     = $data['alias'];

            $city->lat                       = $data['coordinates_lat'];
            $city->lng                       = $data['coordinates_lng'];
            $city->important                 = $data['popular'];

            $city->kladr_country_id          = $region->kladr_country_id;
            $city->kladr_region_id           = $region->id;

            $city->save(false);

            $this->stdoutN("added");
        }

        return $this;
    }

    /**
     * @param $data
     * @param Country $country
     * @return $this
     */
    protected function _addShortRegion($data, Country $country)
    {
        $this->stdoutN("Region: {$data['name']}");

        $region = Region::find()->where(['autoru_id' => $data['id']])->one();

        if ($region)
        {
            $this->stdoutN("Region already exist. Update info.");

            $region->autoru_id                 = $data['id'];
            $region->name                      = $data['name'];
            $region->code                      = $data['code'];
            $region->kladr_country_id          = $country->id;

            $region->save(false);

        } else
        {
            $region = new Region();

            $region->autoru_id                 = $data['id'];
            $region->name                      = $data['name'];
            $region->code                      = $data['code'];
            $region->kladr_country_id          = $country->id;

            $region->save(false);

            $this->stdoutN("Country adding");
        }

        return $this;

    }

    /**
     * @param $data
     * @return array|null|Country
     */
    protected function _addCountry($data)
    {
        $this->stdoutN("Cpuntry: {$data['name']}");

        $country = Country::find()->where(['autoru_id' => $data['id']])->one();

        if (!$country)
        {
            $country = Country::find()->where(['name' => $data['name']])->one();
        }

        if ($country)
        {
            $this->stdoutN("Cpuntry already exist. Update info.");

            $country->autoru_id     = $data['id'];
            $country->name          = $data['name'];
            $country->name_full     = $data['name_full'];
            $country->name_where    = $data['name_where'];
            $country->domain        = $data['domain'];
            $country->dialcode      = $data['dialcode'];
            $country->lat           = ArrayHelper::getValue($data['center'], 'lat');
            $country->lng           = ArrayHelper::getValue($data['center'], 'lng');

            $country->save(false);

        } else
        {
            $country = new Country();

            $country->autoru_id     = $data['id'];
            $country->name          = $data['name'];
            $country->name_full     = $data['name_full'];
            $country->name_where    = $data['name_where'];
            $country->domain        = $data['domain'];
            $country->dialcode      = $data['dialcode'];
            $country->lat           = ArrayHelper::getValue($data['center'], 'lat');
            $country->lng           = ArrayHelper::getValue($data['center'], 'lng');

            $country->save(false);


            $this->stdoutN("Country adding");
        }

        return $country;
    }
}