<?php
/**
 * Updater
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 21.01.2015
 * @since 1.0.0
 */
namespace skeeks\modules\cms\kladr\helpers;

use skeeks\cms\models\Comment;
use skeeks\cms\models\Publication;
use skeeks\cms\models\searchs\Publication as PublicationSearch;
use skeeks\cms\modules\admin\controllers\AdminController;
use skeeks\cms\modules\admin\controllers\AdminModelEditorSmartController;
use skeeks\cms\modules\admin\controllers\AdminModelEditorController;
use skeeks\modules\cms\kladr\models\AdministrativeArea;
use skeeks\modules\cms\kladr\models\Country;
use skeeks\modules\cms\kladr\models\Locality;
use skeeks\modules\cms\kladr\models\Region;
use skeeks\modules\cms\kladr\models\SubAdministrativeArea;
use skeeks\sx\Dir;
use Yii;
use skeeks\cms\models\User;
use skeeks\cms\models\searchs\User as UserSearch;
use yii\base\Component;
use yii\helpers\ArrayHelper;
use yii\helpers\Json;

class Updater extends Component
{
    /**
     * Обновление списка стран
     */
    static public function countriesUpdate()
    {
        $countriesAll = VkApi::getCountries();

        if ($countriesAll['items'])
        {
            foreach ($countriesAll['items'] as $data)
            {
                $yendexGeocode = new YandexGeocode();
                if ($results = $yendexGeocode->request($data['title'])->getResults())
                {
                    $added = false;
                    foreach ($results as $object)
                    {
                        if ($object->getKind() == 'country')
                        {
                            $added = true;
                            $country = Country::find()->where(["name" => $object->getCountryName()])->one();

                            if (!$country)
                            {
                                $country = new Country([
                                    'name' => $object->getCountryName(),
                                    'code' => $object->getCountryNameCode(),
                                    'vk_id' => $data['id']
                                ]);

                                $country->save(false);
                            } else
                            {
                                $country->vk_id = $data['id'];
                                $country->code  = $object->getCountryNameCode();
                                $country->save();
                            }
                        }
                    }
                }

                /*if ($added === false)
                {
                    $country = Country::find()->where(["name" => $data['title']])->one();

                    if (!$country)
                    {
                        $country = new Country([
                            'name' => $data['title'],
                            'code' => null,
                        ]);

                        $country->save(false);
                    }
                }*/
            }
        }
    }

    /**
     * Обновление Регионов
     * @param null $country_id
     */
    static public function regionUpdate($country_id = null)
    {
        if ($id = $country_id)
        {
            $countries = Country::find()
                ->where(['not', ['vk_id' => '']])
                ->andWhere(['id' => $id])
                ->all();
        } else
        {
            $countries = Country::find()->where(['not', ['vk_id' => '']])->all();
        }


        if ($countries)
        {
            foreach ($countries as $country)
            {
                $regionsData = VkApi::getRegions($country->vk_id);
                usleep(100);

                if ($regionsData['items'])
                {
                    foreach ($regionsData['items'] as $regionItem)
                    {
                        $region = Region::find()->where(['name' => $regionItem['title']])->one();

                        if (!$region)
                        {
                            $region = new Region([
                                'name'  => $regionItem['title'],
                                'vk_id' => $regionItem['id'],
                                'kladr_country_id' => $country->id,
                            ]);

                            $region->save(false);
                        } else
                        {
                            $region->vk_id = $regionItem['id'];
                            $region->kladr_country_id = $country->id;
                            $region->save(false);
                        }
                    }
                }
            }
        }

    }


    static public function cityUpdate($country_id = null)
    {
        if ($id = $country_id)
        {
            $countries = Country::find()
                ->where(['not', ['vk_id' => '']])
                ->andWhere(['id' => $id])
                ->all();
        } else
        {
            //$countries = Country::find()->where(['not', ['vk_id' => '']])->all();
        }

        if ($countries)
        {
            foreach ($countries as $country)
            {
                $lastReadyRegion = 0;
                $file = Dir::runtimeTmp()->newFile('test');
                if ($file->isExist())
                {
                    $lastReadyRegion = (int) $file->read();
                }

                $regions = AdministrativeArea::find()->where([
                    'kladr_country_id' => $country->id
                ])->orderBy("id ASC");

                if ($lastReadyRegion)
                {
                    $regions->andWhere("id > '{$lastReadyRegion}'");
                }

                $regions = $regions->all();

                echo "Страна: " . $country->id . " ({$country->name})\n";
                sleep(1);

                if ($regions)
                {
                    foreach ($regions as $region)
                    {
                        echo "Регион: " . $region->id . " ({$region->name}) - vk_id={$region->vk_id}\n";

                        $regionVkData = [];
                        $citiesVkData = VkApi::getCities($country->vk_id, $region->vk_id);

                        $allCount   = ArrayHelper::getValue($citiesVkData, 'count', 0);
                        $items      = ArrayHelper::getValue($citiesVkData, 'items');

                        sleep(1);
                        self::_vkUpdateCities($items, $country, $region);

                        echo "Всего найдено: {$allCount}\n";
                        if ($allCount > 1000)
                        {
                            $pages = $allCount/1000;
                            if ($pages > round($pages))
                            {
                                $pages = round($pages) + 1;
                            }
                            $pages = round($pages);
                        }

                        if ($pages)
                        {
                            for ($i = 1; $i <= $pages; $i++)
                            {
                                echo "Страница: {$i}\n";
                                sleep(1);

                                $founded = 0;
                                $cities = VkApi::getCities($country->vk_id, $region->vk_id, [
                                    'offset' => (int) (1000*$i)
                                ]);

                                $items = ArrayHelper::getValue($cities, 'items');
                                self::_vkUpdateCities($items, $country, $region);
                            }
                        }

                        Dir::runtimeTmp()->newFile('test')->write($region->id);
                    }
                }
            }
        }
    }


    static protected function _vkUpdateCities($items = [], $country, $region)
    {
        ini_set('memory_limit','1024M');
        //Работаем с найдеными направлениями
        foreach ($items as $data)
        {
            $area   = ArrayHelper::getValue($data, 'area');
            $title  = ArrayHelper::getValue($data, 'title');
            $id     = ArrayHelper::getValue($data, 'id');

            if ($area)
            {
                $subArea = SubAdministrativeArea::find()->where([
                    'name'                          => $area,
                    'kladr_country_id'              => $country->id,
                    'kladr_administrative_area_id'  => $region->id
                ])->one();

                if (!$subArea)
                {
                    $subArea = new SubAdministrativeArea([
                        'name'                          => $area,
                        'kladr_country_id'              => $country->id,
                        'kladr_administrative_area_id'  => $region->id,
                    ]);

                    echo "Новый район: " . $area . "\n";
                    $subArea->save(false);
                }
            }

            $locality = Locality::find()->where([
                'name' => $title,
                'kladr_country_id'              => $country->id,
                'kladr_administrative_area_id'  => $region->id
            ])->one();

            if (!$locality)
            {
                echo "Новый город: " . $title . "\n";
                $locality = new Locality([
                    'name'  => $title,
                    'vk_id' => $id,
                    'kladr_country_id' => $country->id,
                    'kladr_administrative_area_id' => $region->id,
                ]);

                if ($subArea)
                {
                    $locality->kladr_sub_administrative_area_id = $subArea->id;
                }

                $locality->save(false);
            } else
            {
                if ($subArea)
                {
                    $locality->kladr_sub_administrative_area_id = $subArea->id;
                }

                $locality->vk_id = $id;
                $locality->kladr_country_id = $country->id;
                $locality->kladr_administrative_area_id = $region->id;
                $locality->save(false);
            }
        }
    }
}