<?php
/**
 * VkApi
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 20.01.2015
 * @since 1.0.0
 */
namespace skeeks\modules\cms\kladr\helpers;

use skeeks\cms\models\Comment;
use skeeks\cms\models\Publication;
use skeeks\cms\models\searchs\Publication as PublicationSearch;
use skeeks\cms\modules\admin\controllers\AdminController;
use skeeks\cms\modules\admin\controllers\AdminModelEditorSmartController;
use skeeks\cms\modules\admin\controllers\AdminModelEditorController;
use skeeks\modules\cms\kladr\models\AdministrativeArea;
use skeeks\modules\cms\kladr\models\Country;
use Yii;
use skeeks\cms\models\User;
use skeeks\cms\models\searchs\User as UserSearch;
use yii\base\Component;
use yii\helpers\Json;

class VkApi extends Component
{
    public $baseApiUrl = 'http://api.vk.com/method/database'; //.getCountries?v=5.5&need_all=1&count=255';
    public $version = '5.5';
    public $headers = [
        'http' => [
            'method' => "GET",
            'header' => 'Cookie: remixlang=0\r\n'
        ]
    ];

    /**
     * @param $methodName
     * @return string
     */
    public function getMethodUrl($methodName)
    {
        return $this->baseApiUrl . '.' . $methodName;
    }

    /**
     * @param $method
     * @param array $queryData
     * @return array
     */
    public function request($method, $queryData = [])
    {
        $requestData = array_merge([
            "v" => $this->version,
        ], $queryData);

        $requestUrl = $this->getMethodUrl($method) . '?' . http_build_query($requestData);
        $json = file_get_contents($requestUrl, false, stream_context_create($this->headers));

        return (array) Json::decode($json)['response'];
    }

    /**
     * @return array
     */
    static public function getCountries()
    {
        return (new self())->request('getCountries', [
            'need_all' => 1,
            'count' => 1000,
        ]);
    }


    /**
     * @param $country_id
     * @return array
     */
    static public function getRegions($country_id)
    {
        return (new self())->request('getRegions', [
            'country_id' => (int) $country_id,
            'need_all' => 1,
            'count' => 1000,
        ]);
    }

    /**
     * @param $country_id
     * @return array
     */
    static public function getCities($country_id, $region_id, $data = [])
    {
        return (new self())->request('getCities', array_merge([
            'region_id'     => (int) $region_id,
            'country_id'    => (int) $country_id,
            'need_all'      => 1,
            'count'         => 1000,
        ], $data ));
    }

}