<?php
/**
 * Select
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 22.01.2015
 * @since 1.0.0
 */
namespace skeeks\modules\cms\kladr\widgets\form;

use skeeks\cms\Exception;
use skeeks\modules\cms\kladr\models\City;
use skeeks\modules\cms\kladr\models\Metro;
use skeeks\modules\cms\kladr\models\Region;
use skeeks\modules\cms\kladr\models\Road;
use skeeks\modules\cms\kladr\widgets\form\assets\SelectAsset;
use yii\helpers\ArrayHelper;
use yii\helpers\Html;
use yii\helpers\Json;
use yii\widgets\InputWidget;
use Yii;

/**
 * Class Select
 * @package skeeks\modules\cms\kladr\widgets\form
 */
class Select extends InputWidget
{
    /**
     * @var array the options for the Bootstrap File Input plugin. Default options have exporting enabled.
     * Please refer to the Bootstrap File Input plugin Web page for possible options.
     * @see http://plugins.krajee.com/file-input#options
     */
    public $clientOptions = [];

    public $fieldNameCountry      = 'kladr_country_id';
    public $fieldNameRegion       = 'kladr_region_id';
    public $fieldNameCity         = 'kladr_city_id';
    public $fieldNameMetro        = 'kladr_metro_id';
    public $fieldNameRoad         = 'kladr_road_id';

    public $allowFields =
    [
        'kladr_country_id', 'kladr_region_id', 'kladr_city_id', 'kladr_metro_id', 'kladr_road_id'
    ];

    /**
     * Берем поведения модели
     *
     */
    private function _initAndValidate()
    {
        if (!$this->hasModel())
        {
            throw new Exception("Этот файл рассчитан только для форм с моделью");
        }
    }

    /**
     * @inheritdoc
     */
    public function run()
    {
        try
        {
            $regionItems = [];
            if ($this->model->hasAttribute($this->fieldNameCountry))
            {
                if ($countryId = $this->model->{$this->fieldNameCountry})
                {
                    $regionItems = \yii\helpers\ArrayHelper::map(
                        Region::find()->where(['kladr_country_id' => $countryId])->all(),
                        'id', 'name'
                    );
                }
            }

            $cityItems = [];
            if ($this->model->hasAttribute($this->fieldNameRegion))
            {
                if ($regionId = $this->model->{$this->fieldNameRegion})
                {
                    $cityItems = \yii\helpers\ArrayHelper::map(
                        City::find()->where(['kladr_region_id' => $regionId])->all(),
                        'id', 'name'
                    );
                }
            }


            $metroItems = [];
            if ($this->model->hasAttribute($this->fieldNameCity))
            {
                if ($cityId = $this->model->{$this->fieldNameCity})
                {
                    $metroItems = \yii\helpers\ArrayHelper::map(
                        Metro::find()->where(['kladr_city_id' => $cityId])->all(),
                        'id', 'name'
                    );
                }
            }


            $roadItems = [];
            if ($this->model->hasAttribute($this->fieldNameCity))
            {
                if ($cityId = $this->model->{$this->fieldNameCity})
                {
                    $roadItems = \yii\helpers\ArrayHelper::map(
                        Road::find()->where(['kladr_city_id' => $cityId])->all(),
                        'id', 'name'
                    );
                }
            }


            $this->_initAndValidate();
            $this->registerClientScript();

            $id         = "sx-id-" . Yii::$app->security->generateRandomString(6);

            $clientOptions = ArrayHelper::merge($this->clientOptions, [
                'fieldNameCountry'    => $this->fieldNameCountry,
                'fieldNameRegion'     => $this->fieldNameRegion,
                'fieldNameCity'       => $this->fieldNameCity,
                'fieldNameMetro'      => $this->fieldNameMetro,
                'fieldNameRoad'       => $this->fieldNameRoad,


                'fieldNameCountryId'        => Html::getInputId($this->model, $this->fieldNameCountry),
                'fieldNameRegionId'         => Html::getInputId($this->model, $this->fieldNameRegion),
                'fieldNameCityId'           => Html::getInputId($this->model, $this->fieldNameCity),
                'fieldNameMetroId'          => Html::getInputId($this->model, $this->fieldNameMetro),
                'fieldNameRoadId'           => Html::getInputId($this->model, $this->fieldNameRoad),

                'regionsBackendUrl'      => \skeeks\cms\helpers\UrlHelper::construct('kladr/ajax/regions')->toString(),
                'citiesBackendUrl'       => \skeeks\cms\helpers\UrlHelper::construct('kladr/ajax/cities')->toString(),
                'metroBackendUrl'        => \skeeks\cms\helpers\UrlHelper::construct('kladr/ajax/metro')->toString(),
                'roadsBackendUrl'        => \skeeks\cms\helpers\UrlHelper::construct('kladr/ajax/roads')->toString(),
            ]);

            return $this->render('select', [
                'widget'            => $this,
                'id'                => $id,
                'model'             => $this->model,
                'clientOptions'     => $clientOptions,

                'fieldNameCountryId'       => Html::getInputId($this->model, $this->fieldNameCountry),
                'fieldNameRegionId'         => Html::getInputId($this->model, $this->fieldNameRegion),
                'fieldNameCityId'           => Html::getInputId($this->model, $this->fieldNameCity),
                'fieldNameMetroId'          => Html::getInputId($this->model, $this->fieldNameMetro),
                'fieldNameRoadId'           => Html::getInputId($this->model, $this->fieldNameRoad),

                'regionItems'           => $regionItems,
                'cityItems'             => $cityItems,
                'metroItems'            => $metroItems,
                'roadItems'             => $roadItems,
            ]);

        } catch (Exception $e)
        {
            echo $e->getMessage();
        }

    }

    /**
     * Registers Bootstrap File Input plugin
     */
    public function registerClientScript()
    {
        $view = $this->getView();
        SelectAsset::register($view);
    }
}
